SystemOrganization addCategory: #'Magma seaside'!

TestCaseUsingMagma subclass: #WAReadStrategyClassSpecTest
	instanceVariableNames: 'rs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAReadStrategyClassSpecTest methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 03:42'!
readStrategyClass

	^ WAMagmaReadStrategyDefault! !

!WAReadStrategyClassSpecTest methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:03'!
setUp

	self magma initializeRoot: Dictionary new.! !

!WAReadStrategyClassSpecTest methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 05:34'!
testSimpleCase

	self magma session commit: [ self magma session root at: 'TestCase' put: TestCase new ].

	rs := self readStrategyClass minimumDepth: 1.
 
	self magma session readStrategy: rs. 



	"

	self debug: #testSimpleCase


	"! !

MaReadStrategy subclass: #WAMagmaReadStrategyDefault
	instanceVariableNames: 'session fullSpecifications'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaReadStrategyDefault commentStamp: 'kph 6/14/2007 10:56' prior: 0!
This class is a subclass of MaReadStrategy enabling specialisation of read strategy behaviour.

WAMagmaSession asks us to build a readStrategy, our default is none (subclasses can override #buildThisStrategyOn:), we then ask 
the session to #buildReadStrategyOn: and the sessions default behavour is to ask the root object to #buildReadStrategyOn: 

This allows each application have its own strategy defined by its root object without needing any specialized sessions or strategies.

If you have a specialized readstrategy, or a specialised session you can override this behaviour.

Alternative admin-user selectable (via seaside/config) read strategies may be supplied by providing subclasses to this class. These may override the default building behaviour if desired. They may also over-ride MaReadStrategy behaivour.

In seaside the Session readStrategy is updated/rebuilt whenever there is a new seaside session, even when the MagmaSession and the repository is shared among server seaside sessions.

There is an enhancement to read strategies:

1. specify a default depth AND individual depths for particular instVars
2. Negative values are valid and are subtracted from the inherited/active value
 


!

!WAMagmaReadStrategyDefault class methodsFor: 'as yet unclassified' stamp: 'kph 6/12/2007 07:07'!
buildThisStrategyOn: strategy

"subclasses may specialise this method"! !

!WAMagmaReadStrategyDefault class methodsFor: 'as yet unclassified' stamp: 'kph 9/20/2006 05:56'!
isLogger

^false! !

!WAMagmaReadStrategyDefault class methodsFor: 'as yet unclassified' stamp: 'kph 6/13/2007 20:33'!
setStrategyUsing: builder in: session

| theStrategy |

theStrategy := self new.
theStrategy session: session.

self buildThisStrategyOn: theStrategy.
builder buildReadStrategyOn: theStrategy. 
 
^theStrategy! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 04:30'!
convertSpecificationsToIdsUsing: aMaClassIdManager 
	"Convert the classes in the receiver to classId's, since that is what the server goes by."
	| specifiedClassNames |
	specifiedClassNames _ depthSpecifications keys collect: [ : each | each name ].

	aMaClassIdManager classDefinitionsById do: 
		[ : eachDefinitions | 
		eachDefinitions do: 
			[ : eachDefinition | 
			"Check superclasses to inherit their specification, but only if one was not already specified for eachDefinition."
			eachDefinition withAllSuperclassDefinitionsDo: 
				[ : eachDef | 
				(specifiedClassNames includes: eachDef name) ifTrue: 
					[ depths 
						at: { eachDefinition id. eachDefinition version }
						ifAbsentPut: 
							[ self beNew.
							self instVarMapFrom: eachDef for: eachDefinition ] ] ] ] ]
	! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 01:18'!
depthArrayFor: aMaClassDefinition defaultDepth: defaultDepth


	"Given 

	MySuperclass with instvers: 'a b c'
	MySubclass with instvars: 'd e f'

	we turn onAny: MySubclass defaultDepth: 3

	into #(nil nil nil 3 3 3) "


	| result nInherited |

	result := Array new: aMaClassDefinition namedInstSize.

	nInherited := aMaClassDefinition superclassDefinition namedInstSize.

	nInherited to: result size do: [ :n |
	
		result at: n put: defaultDepth
	].

	^ defaultDepth! !

!WAMagmaReadStrategyDefault methodsFor: 'compatability' stamp: 'kph 6/19/2007 01:51'!
forVariableNamed: aString onAny: aClass readToDepth: anInteger

	"compatibility"

	(self onAny: aClass) readVar: aString toDepth: anInteger! !

!WAMagmaReadStrategyDefault methodsFor: 'compatability' stamp: 'kph 6/19/2007 01:51'!
forVariablesNamed: aString onAny: aClass readToDepth: anInteger

	"compatibility"

	(self onAny: aClass) readVars: aString toDepth: anInteger! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 06:42'!
inheritDepthsUsing: aMaClassIdManager 
	 

	depths do: [ :ea | ea inheritDepthsIn: depths using: aMaClassIdManager ].
			 
			! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/13/2007 20:42'!
initialize

	super initialize.
	self reset! !

!WAMagmaReadStrategyDefault methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:31'!
instVarMapFrom: aMaClassDefinition for: aMaSubclassDefinition 
	"Answer an Array with an instVarMap of aMaClassDefinition according to user-declared read depths."
	| |

	(aMaClassDefinition isDefined and: [ depthSpecifications includesKey: aMaClassDefinition classObject ]) 
		ifFalse: [ ^ nil ].

	^ (depthSpecifications at: aMaClassDefinition classObject) copy makeInstVarMapFor: aMaSubclassDefinition! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 9/20/2006 15:24'!
isLogged

^self class isLogger! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 23:25'!
makeReadyForUseUsing: aMaClassIdManager

	self 
		resetDepths ;
		convertSpecificationsToIdsUsing: aMaClassIdManager.
	self inheritDepthsUsing: aMaClassIdManager.
	fullSpecifications := depths copy. "make a copy of the fully processed specs for debugging etc"
	
	self tidyUpDepths.! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 04:07'!
onAny: aClass
 
	^ depthSpecifications 
		at: aClass
		ifAbsentPut: 
			[ WAReadStrategyClassSpec forClass: aClass ]! !

!WAMagmaReadStrategyDefault methodsFor: 'compatability' stamp: 'kph 6/19/2007 01:52'!
onAny: aClass readToDepth: anInteger 

	(self onAny: aClass) readAllVarsToDepth: anInteger! !

!WAMagmaReadStrategyDefault methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 13:43'!
renderOn: html

html text: ' minimumDepth: '; text: self minimumDepth; break.

depthSpecifications keysAndValuesDo: [ :k :v |  html strong: k; text: ' : '; text: v; break ].! !

!WAMagmaReadStrategyDefault methodsFor: 'as yet unclassified' stamp: 'kph 6/13/2007 21:34'!
reset
! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 23:26'!
resetSpecifications

	depthSpecifications := nil.
	fullSpecifications := nil.! !

!WAMagmaReadStrategyDefault methodsFor: 'as yet unclassified' stamp: 'kph 6/18/2007 08:09'!
session: ms

	"for logging, it helps to have the session since the session can convert id's to class names"
	session := ms! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 05:12'!
tidyUpDepths

	depths keys do: [ :key |
		depths at: key put: (self tidyUpDepths: (depths at: key))
	].! !

!WAMagmaReadStrategyDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 06:43'!
tidyUpDepths: spec

	| a array |

	array := spec instVarMap.
	
	array isEmpty ifTrue: [ ^ 1 ].

	array detect: [ :ea | (a ifNil: [a := ea ])  ~= ea ] ifNone: [ ^ a ].

	^ array! !

WAMagmaReadStrategyDefault subclass: #WAMagmaReadStrategyLogged
	instanceVariableNames: 'firstTime readStart totalTime timeStamp readCount entryIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaReadStrategyLogged commentStamp: 'kph 6/13/2007 20:41' prior: 0!
This class uses Q2Log from the Gjallar package to log to the Transcript or to a file!

!WAMagmaReadStrategyLogged class methodsFor: 'as yet unclassified' stamp: 'kph 6/13/2007 18:19'!
isLogger

^true! !

!WAMagmaReadStrategyLogged methodsFor: 'as yet unclassified' stamp: 'kph 6/13/2007 20:20'!
log

^ Q2Log! !

!WAMagmaReadStrategyLogged methodsFor: 'as yet unclassified' stamp: 'kph 6/13/2007 20:06'!
reset

	firstTime := nil.
	entryIndex := 0.! !

!WAMagmaReadStrategyLogged methodsFor: 'as yet unclassified' stamp: 'kph 6/14/2007 01:01'!
signalBeginningToRead: oidInteger classId: classId

"context expected for
entryForReading: oid at: currentDepth to: minDepth attribute: varIndex ofClassId: classId version: classVersion 
"
	| theValue   theClass |

	readStart ifNotNil: [ totalTime := totalTime + timeStamp - readStart ].
					
	readStart := timeStamp := Time millisecondClockValue.

	firstTime ifNil: [ firstTime := readStart. readCount  := 0. totalTime := 0 ].
	readCount := readCount + 1.
	
	self log debug: '-- ', readCount asString, ' --' sender: ''.
	
	theClass := session serializer classIdManager classForId: classId ifAbsent: [ ^self error: 'could not find class id' ].

	theValue := session serializer objectWithOid: oidInteger.
	theValue maIsMutatingProxy ifTrue: [ theValue := (theClass name, ' oid:', (oidInteger printString))].

	self log debug: (' [', totalTime asString, 'ms] Reading ') , (theValue printStringLimitedTo: 70) sender: ''. 

	 
  

 

! !

!WAMagmaReadStrategyLogged methodsFor: 'as yet unclassified' stamp: 'kph 6/22/2007 16:25'!
testRead: oid 
    wantedAt: currentDepth
		   to: minDepth
 forAttribute: varIndex
ofClassWithId: classId
	  version: classVersion
 
| newMinDepth  theClass theAttributeName theValue out |					
		
timeStamp := Time millisecondClockValue.

newMinDepth :=  super testRead: oid wantedAt: currentDepth
						to: minDepth
						forAttribute: varIndex
						ofClassWithId: classId
						version: classVersion.

newMinDepth > currentDepth ifFalse: [ ^ newMinDepth ].
 
entryIndex := entryIndex + 1.
 
theClass := session serializer classIdManager classForId: classId ifAbsent: [ ^self error: 'could not find class id' ].
 
theAttributeName := theClass allInstVarNames at: varIndex  ifAbsent: [ varIndex asString ].
 
theValue := session serializer objectWithOid: oid.  
theValue maIsMutatingProxy 
	ifTrue: [ theValue := ('oid:',oid printString)]
	ifFalse: [ theValue := theValue class name, ' ',  (theValue  printStringLimitedTo: 40) ].

out :=  (String streamContents: [ :stream | 

stream 	nextPutAll: entryIndex asString;
		nextPutAll: ': [';
		nextPutAll: (totalTime + timeStamp - readStart) asString;
		nextPutAll: 'ms] (';
		nextPutAll: currentDepth asString;
		nextPut: $/;
		nextPutAll: minDepth asString.

 
stream		nextPut: $/;
		nextPutAll: (newMinDepth) asString.

stream		nextPutAll: ') ';
		nextPutAll: theClass name;
		nextPut: $-;
		nextPutAll: theAttributeName;
		nextPutAll: ' = ';
		nextPutAll:  theValue  ;
		nextPutAll: ((newMinDepth == false) ifTrue: [ ' X' ] ifFalse: [''])
		yourself.	
]).

self log debug: out sender: ''.

^newMinDepth ! !

!MaObjectBuffer methodsFor: '*magma seaside-override' stamp: 'kph 7/18/2007 11:49'!
appendGraphUsing: aMaReadStrategy into: aMaSerializedGraphBuffer currentDepth: currentDepth minDepth: minDepth with: alreadyAppendedSet filer: aMaObjectFiler

	| classId classVersion |

	classId := self classId.
	classVersion := self classVersion.

	1
		to: self objectInstSize
		do:
			[ :varIndex | | oid |

			((MaOidCalculator isOidForOptimizedObject: (oid _ self maInstVarAt: varIndex)) 
			)

				ifFalse:
					[ | newMinDepth | 
			 
				(newMinDepth := aMaReadStrategy 
						testRead: oid
						wantedAt: currentDepth
						to: minDepth
						forAttribute: varIndex
						ofClassWithId: classId
						version: classVersion) > currentDepth  ifTrue: [
						
							self
								appendGraphNode: oid
								into: aMaSerializedGraphBuffer
								using: aMaReadStrategy
								currentDepth: currentDepth
								minDepth: newMinDepth
								with: alreadyAppendedSet
								filer: aMaObjectFiler 
							].
					  ]
		]
 ! !

WAConfigurationAttribute subclass: #WAMagmaLocationAttribute
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaLocationAttribute methodsFor: 'as yet unclassified' stamp: 'brp 11/4/2005 10:00'!
accept: aVisitor with: anObject

	^ aVisitor visitMagmaLocationAttribute: self with: anObject! !

!WAMagmaLocationAttribute methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 11:55'!
options

	^ MagmaLocation allSubclasses! !

!MagmaRepositoryController class methodsFor: '*magma seaside' stamp: 'kph 8/24/2006 22:21'!
openControllers

^OpenControllers! !

!MaBasicReadStrategy methodsFor: '*magma seaside' stamp: 'kph 6/14/2007 00:02'!
signalBeginningToRead: oidInteger classId: classId! !

!MaBasicReadStrategy methodsFor: '*magma seaside' stamp: 'kph 6/22/2007 16:16'!
testRead: oid wantedAt: currentDepth
			to: minDepth
			forAttribute: varIndex
			ofClassWithId: classId
			version: classVersion

| newMinDepth attrDepth |

	attrDepth := self 
						depthOfAttribute: varIndex
						onClassWithId: classId
						version: classVersion.
	newMinDepth := 
		attrDepth > 0 	ifTrue: [ attrDepth + currentDepth max: minDepth ]
						ifFalse: [ currentDepth max: (minDepth + attrDepth "negative") ].
	
	
	^ newMinDepth
! !

!WAConfigurationEditor methodsFor: '*magma seaside' stamp: 'kph 5/24/2007 11:54'!
renderMagmaLocationAttribute: anAttribute on: html

	| location |
	location := configuration valueForAttribute: anAttribute.

	html div id: 'location'; with: [
		html table: [
			(location respondsTo: #comment) ifTrue: [ html small: location comment ].
			location isLocal
				ifTrue: [
					html tableRow: [  
						html tableData id: 'label'; with: 'path:'.
						html tableData: [ html textInput on: #path of: location ]
					]
				] ifFalse: [
					html tableRow: [  
						html tableData id: 'label'; with: 'host'.
						html tableData: [ html textInput on: #host of: location ]
					].
					html tableRow: [  
						html tableData id: 'label'; with: 'port'.
						html tableData: [ html textInput value: location port;
								callback: [:v | location port: v asNumber] ]
					].
				]
		]
	].
! !

!WAConfigurationEditor methodsFor: '*magma seaside' stamp: 'kph 6/15/2006 20:33'!
visitMagmaLocationAttribute: anAttribute with: html

	| location |
	location := configuration valueForAttribute: anAttribute. 

	html select 
		list: anAttribute options;
		selected: location class;
		beSubmitOnChange;
		labels: [ :ea | ea name ];
		callback: [ :item | 	(configuration valueAt: #location) class == item
			ifFalse: [ configuration takeValue: item default forAttribute: anAttribute. ]
 		];
		yourself.

	self renderMagmaLocationAttribute: anAttribute on: html.! !

WADispatcherControlPanel subclass: #WAMagmaControlPanel
	instanceVariableNames: 'repositoryControlPanel textInput doneIt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaControlPanel commentStamp: 'kph 4/19/2007 23:28' prior: 0!
Architecture:

This control panel is made available as a plug-in to the seaside/config disptacher editor. The link 'magma' appears next to applications that have the group #magma. Since the seaside/config application session is not, nor should it be a WAMagmaSession, this control panel has to manage its own magmaSession. It does this by creating a psuedo seaside application, local to this control panel which has a magmaSession. It also registers for finalization in order to safely dispose of the psuedo application and to disconnect the magmaSession.

This control panel anticipates that each application using magma may have its own repository control/status panel class. The root object of the repository is sent #magmaControlPanelOn: <magmaSession> to obtain a control/status panel component, or a descriptive string. This is displayed as a sub-part of the WAMagmaControlPanel.

If there is no repository in place then the user is offered the option to create one. It searches for the repository control panels.  These are seaside-components which implement #magmaRootClass and #magmaRootDescription, sufficient information for the user to select a root object that is appropriate for their application.

[to do]
seasidehosting currently requires the user to manually create the repository directory in the filesystem.



!

!WAMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 02:12'!
initialize
	"for seaside version<=2.7"
 
	 ^ self register! !

!WAMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 02:06'!
label
	"for seaside version<=2.7"
 
	 ^ 'Magma'! !

!WAMagmaControlPanel methodsFor: 'all stats' stamp: 'kph 9/18/2006 04:18'!
cachedObjectsByClassForSession: aSession

^String streamContents: [ :stream |
	stream nextPutAll: 'cachedObjectCountByClass: '.
	aSession cachedObjectCountByClass 
		do: [ :assoc |
			stream nextPutAll: assoc printString.
			stream nextPutAll: ' '; cr.
		]
]! !

!WAMagmaControlPanel methodsFor: 'all stats' stamp: 'kph 7/26/2007 22:16'!
children

^ Array with: repositoryControlPanel  ! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 7/26/2007 22:17'!
createRepository: repositoryClass
	self appSession unregistered.
	appSession := nil.
	repositoryControlPanel := null.
	
 	MagmaRepositoryController create: self location root: repositoryClass new.
! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 7/26/2007 22:18'!
deleteDatabase

| message1 message2 |

message1 := 'Are you sure that you want to delete repository: ', self location path.
message2 := 'Go ahead and delete repository: ',  self location path.

((self confirm: message1) and: [ self confirm: message2 ]) 
	ifTrue: [ 
		self psuedoAppSession expire.
		appSession := nil.
		repositoryControlPanel := null.
		self location delete 
	].! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 9/4/2006 03:15'!
doIt: aString

doneIt := ( Compiler new evaluate: aString in: nil to: nil ) printString.
! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 9/18/2006 04:24'!
explore: aMagmaSession
 
 [ { aMagmaSession. aMagmaSession root } explore ] fork! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/23/2006 07:02'!
explore: sessionId inController: mrc

	self explore:(self findMagmaSessionWithId: sessionId inController: mrc)! !

!WAMagmaControlPanel methodsFor: 'all stats' stamp: 'kph 9/20/2006 15:05'!
findMagmaSessionWithId: id inController: mrc

^MagmaSession allInstances detect: [ :ms | (ms id = id) and: [ ms link localRequestServer processor = mrc ] ] ifNone: [ nil ]! !

!WAMagmaControlPanel methodsFor: 'accessing' stamp: 'kph 7/27/2007 02:02'!
initialize

	super initialize.
	repositoryControlPanel := null.! !

!WAMagmaControlPanel methodsFor: 'accessing' stamp: 'kph 9/18/2006 01:27'!
location

^ self application preferenceAt: #location! !

!WAMagmaControlPanel methodsFor: 'accessing' stamp: 'kph 8/2/2007 18:49'!
magmaSession

	(self appSession respondsTo: #magmaSession) ifFalse: [ self notify: 'Configured session class does not support magma'].
	
	^ self appSession magmaSession
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 8/2/2007 18:56'!
magmaSession: aBlock ifAbsent: notThereBlock

aBlock
	on: MagmaEnvironmentError, InvalidDirectoryError
	do: notThereBlock.  ! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:41'!
presentCreateRepositoryOptionsOn: html

	html heading level: 3; with: [ html text:  'options:'].
		html text: 'You may create a repository with one of the following root objects for this application.'.
		html div class: 'repositories'; with: [ 
			html table: [
				((SystemNavigation default allImplementorsOf: #magmaRootClass) collect: [ :m | m actualClass soleInstance ])				
				do: [ :c | 
						html tableRow: [ 
					 			html tableData: [ html strong:  c magmaRootClass name ].
								html tableData: [ html text:   c magmaRootDescription ].
								html tableData: [ html submitButton callback: [ self createRepository: c magmaRootClass ]; text: 'Create'. ]
						] 
					]
				]
			].! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 7/11/2007 17:20'!
renderApplicationStatusOn: html

	| readStrategy |

	self magmaSession.
 	self renderLocationOn: html.
	self location isLocal ifTrue: [  html text: '   '. 
		html small: [ 
			html anchor 
				callback: [ self deleteDatabase ]; 
				title: 'delete the repository from disk';
				with: '(delete)'. 
		]
	].

	html break.

	html strong: 'Persisted Objects Count: '.
	html text: self magmaSession repositoryController objectCount.

	html break.

	html strong: 'Seaside Session: '.
	html text: self appSession printString.

	html break.

	html strong: 'Session: '.
	html text: self magmaSession printString.

	html break.

	html strong: 'Session Id: '.
	html text: self magmaSession id.

	html break.

	html strong: 'Cached Object Count: '.
	html text: self magmaSession cachedObjectCount.

	html break.

	readStrategy := self magmaSession readStrategy.

	html strong: 'ReadStrategy: '.
	html text: (readStrategy ifNil: [ 'No Read Strategy Set' ]) asString.

	
	((readStrategy respondsTo:  #isLogged) and: [ readStrategy isLogged ]) ifTrue: [
			html anchor callback: [ " self call: (WAMagmaLog log forApplication: application) " ]; with: '(log)'.
		].

	html break.

	html strong: 'Root Object: '.
	html text: self magmaSession root asString.
	
	html text: ' '.
	html small: [ 
		html anchor callback: [ self explore: self magmaSession ]; 
			title: 'open explorer in squeak';
			with: '(explore)'.
			.
	].
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 7/27/2007 02:04'!
renderContentOn: html

	super renderContentOn: html.
	
	html form: [

		self magmaSession: [ self renderApplicationStatusOn: html.
				html horizontalRule.
				html render: self repositoryControlPanel.
				html horizontalRule. 
			] 
			ifAbsent: [ :ex | self renderMagmaError: ex on: html ].

		html break.
		html submitButton on: #done of: self.

		html horizontalRule.

		html heading level2; with: 'Overall Magma Status'.
	 
		self renderMagmaPreferencesOn: html.
		self renderStatsForAllRepositoriesOn: html.
	
		self renderUtilsOn: html.

	].! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/4/2006 02:01'!
renderEvaluatorOn: html
	self
		renderLabel: 'Text-Input'
		control: [ :fid :mid |
			html textInput
				value: textInput;
				callback: [ :value | textInput := value ].
			html space.
			html submitButton
				onClick: (html updater
					id: mid;
					triggerForm: fid;
					callback: [ :r | r render: textInput ];
					return: false);
				text: 'Submit' ]
		model: textInput
		on: html! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/4/2006 02:13'!
renderLabel: aString control: aBlock model: anObject on: html
	| fid mid |
	fid := mid := nil.
	html div class: 'row'; with: [
		html div class: 'label'; with: aString.
		html div class: 'control'; with: [
			html form
				id: (fid := html nextId);
				with: [ aBlock value: fid value: (mid := html nextId) ] ].
		html div class: 'model'; id: mid; with: anObject.
		html div class: 'clear' ]! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:42'!
renderLocationOn: html

	self location isLocal 
		ifTrue: [
			html strong: 'Repository Path Setting: '.
			html text: self location path.
 		]
		ifFalse: [ 
			html strong: 'Repository remote host: '.
			html text: self location host , ':' , self location port
		].
		
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:42'!
renderMagmaError: ex on: html

	self renderLocationOn: html.
	html break.
	html strong: 'Error: ', ex messageText.
	html break.
	self location isLocal 
		ifTrue: [ self renderPresentCreateRepositoryOptionsOn: html	] 
		ifFalse: [ html text: 'Remote repository not available' ].! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:42'!
renderMagmaPreferencesOn: html

html strong: 'debug proxies '.
html checkbox title: 'print shows proxy'; on: #debugProxies of: MagmaPreferences.
html strong: 'hand status '.
html checkbox title: 'show wait cursor when submitting magma requests'; on: #handStatus of: MagmaPreferences.
html strong: 'debug '.
html checkbox title: 'onError Halt, and 2 day server link timeout'; on: #debug of: MagmaPreferences.! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:42'!
renderPresentCreateRepositoryOptionsOn: html

	html heading level: 3; with: [ html text:  'options:'].
		html text: 'You may create a repository with one of the following root objects for this application.'.
		html div class: 'repositories'; with: [ 
			html table: [
				((SystemNavigation default allImplementorsOf: #magmaRootClass) collect: [ :m | m actualClass soleInstance ])				
				do: [ :c | 
						html tableRow: [ 
					 			html tableData: [ html strong:  c magmaRootClass name ].
								html tableData: [ html strong:   c magmaRootDescription ].
								html tableData: [ html submitButton callback: [ self createRepository: c magmaRootClass ]; text: 'Create'. ]
						] 
					]
				]
			].! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/4/2006 03:16'!
renderServerDoItOn: html
	self
		renderLabel: 'Server Do It'
		control: [ :fid :mid |
			html textInput
				value: textInput;
				callback: [ :value | self doIt: (textInput := value).  ].
			html space.
			html submitButton
				onClick: (html updater
					id: mid;
					triggerForm: fid;
					callback: [ :r | r render:textInput ];
					return: false);
				text: 'do it' ]
		model: doneIt 
		on: html! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 6/13/2007 19:45'!
renderStatsForAllRepositoriesOn: html

| mSession |

html orderedList: [ 
	MagmaRepositoryController openControllers do: [ :mrc |
		
		html listItem: [
			html strong: mrc objectCount.
			html text: ' objects in repository: '.
			 
			 html strong: mrc pathName.
			html orderedList: [
				mrc repository sessions do: [ :client |
					
					mSession := self findMagmaSessionWithId: client sessionId inController: mrc.

					html listItem: [
						mSession ifNil: [ html text: 'session not found' ]
							ifNotNil: [ 
								html strong: 'session id: '.
								html text: client sessionId printString.
								html strong: ' cached objects: '.
								html anchor callback: [ self showCachedObjectsByClassFor: client sessionId inController: mrc ]; 
								with: mSession cachedObjectCount.
							html text: ' '.
							html small: [ html anchor callback: [ self explore: client sessionId inController: mrc ]; with: '(explore)' ].
						]
					].
				]	 
			].			
				"html tableData: [ html text: c numberOfConnectedClients asString ]."
		].
	].
]
	 
 
 


 ! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:43'!
renderUtilsOn: html

html horizontalRule.

html strong: 'Miscellaneous Utilities: '.

html break.

html submitButton callback: [ MagmaSession cleanUp.
							  repositoryControlPanel := nil ]; text: 'MagmaSession cleanUp'.

html break.

html submitButton callback: [ WAMagmaSharedSession initialize. ]; title: 'New Sessions (Reloads ReadStrategies)';  text: 'Reset WAMagmaSharedSession'.

html break.


html submitButton callback: [ self saveImageSnapshot. self magmaSession begin ]; text: 'Save Server Image'.

self hasScriptaculous ifTrue: [ html horizontalRule.  self renderServerDoItOn: html ].! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 8/10/2006 16:38'!
rendererClass
        ^ WARenderCanvas ! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/18/2006 04:41'!
repositoryControlPanel

| root |

repositoryControlPanel ifNil: [
	root := self magmaSession root.
	(root respondsTo: #magmaControlPanelOn: ) 
		ifTrue: [ repositoryControlPanel := root magmaControlPanelOn: self magmaSession ]
].

^repositoryControlPanel! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 9/4/2006 01:43'!
saveImageSnapshot

 SmalltalkImage current snapshot: true andQuit: false.! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/23/2006 07:00'!
showCachedObjectsByClassFor:  sessionId inController: mrc 

self inform: (self cachedObjectsByClassForSession: (self findMagmaSessionWithId: sessionId inController: mrc))! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 8/11/2006 14:16'!
style

^ 

'
div.repositories table td {
	border: 1px solid #888;

'! !

Object subclass: #WAReadStrategyClassSpec
	instanceVariableNames: 'theClass myDefinition superSpec specificVars myVarsDepth allVarsDepth instVarMap inheritedExtraVarsDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAReadStrategyClassSpec commentStamp: 'kph 6/19/2007 01:35' prior: 0!
The antidote to the brain bending default implementation!

!WAReadStrategyClassSpec class methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:07'!
forClass: aClass

	^ (self basicNew) setClass: aClass; initialize; yourself! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 06:15'!
allVarsDepth

	^ allVarsDepth! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 06:38'!
extraVarsDepth

	^ (myVarsDepth ifNil: [ allVarsDepth ]) ifNil: [ inheritedExtraVarsDepth ]! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 14:31'!
inheritDepthsIn: depths using: aMaClassIdManager

	| inheritedDepths superClassDepths |

 
	" Q2Log debug:  self printString. "

	"if we have an answer we dont need to calc one" 
	instVarMap ifNotNil: [ ^ self ].

	"if we have no superclass then use our result"
	
	superClassDepths := depths at: { myDefinition superclassDefinition id.  myDefinition superclassDefinition version } ifAbsent: [ self  ].

	"we are going to inherit from our superclass we make sure that he has inherited"

	superClassDepths ~= self ifTrue:[ superClassDepths inheritDepthsIn: depths using: aMaClassIdManager].
	
	inheritedDepths := superClassDepths instVarMap.
	inheritedExtraVarsDepth := superClassDepths extraVarsDepth.
	 
	instVarMap := self instVarMap.

	instVarMap withIndexDo: [ :d :n | 
		(instVarMap at: n) ifNil: [ 
			instVarMap at: n put: (((inheritedDepths at: n ifAbsent:[nil]) ifNil: [ self extraVarsDepth ]) )
		]
	].


	(instVarMap  includes: nil) ifTrue: [ self error: 'incomplete read strategy' ].
	! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 06:37'!
inheritedExtraVarsDepth: evd

	 inheritedExtraVarsDepth := evd! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 07:10'!
instVarMap

	instVarMap ifNotNil: [ ^ instVarMap ].

	instVarMap := Array new: myDefinition namedInstSize.

	allVarsDepth ifNotNil: [ 
		instVarMap isEmpty ifTrue: [ ^instVarMap := Array with: allVarsDepth ].

		instVarMap atAllPut: allVarsDepth 
	].

	myVarsDepth ifNotNil: [ 
		myDefinition superclassDefinition namedInstSize + 1 to: instVarMap size do: [ :n |
			instVarMap at: n put: myVarsDepth 
		].
 	].

	specificVars ifNotNil: [ 
		specificVars keysAndValuesDo: [ :var :depth |
			instVarMap at: (myDefinition allInstVarNames indexOf: var) put: depth
		]
	].

	^ instVarMap! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:42'!
isArray

	^ false! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:33'!
makeInstVarMapFor: aClassDefinition

	self setClassDefinition: aClassDefinition.
	"we dont make the map yet, since we have it in mind!!"

	^ self ! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 7/12/2007 16:57'!
printOn: str
	
	myDefinition ifNotNil: [ str	print: { myDefinition className. myDefinition version.} ; nextPutAll: '->';   space ].
	str print: theClass.
	allVarsDepth ifNotNil: [ str nextPutAll: ' all:'; print: allVarsDepth  ].
	myVarsDepth ifNotNil: [ str nextPutAll: ' my:'; print: myVarsDepth; space  ].
	specificVars ifNotNil: [ str print: specificVars ].
	instVarMap ifNotNil: [ str space; print: instVarMap ].

	self extraVarsDepth ifNotNil: [ str nextPutAll: 'scExtras:'; print: self extraVarsDepth; cr ].! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 01:37'!
readAllVarsToDepth: d

	allVarsDepth := d! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 01:37'!
readMyVarsToDepth: d

	myVarsDepth := d! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/22/2007 16:52'!
readVar: var toDepth: d

	(theClass allInstVarNames includes: var) ifFalse: 
		[ MagmaUserError signal: var , ' is not an attribute of ' , var name ].

	(specificVars ifNil: [ specificVars := Dictionary new ]) at: var put: d

	! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 05:37'!
readVars: vars toDepth: d

	(vars findTokens: ' ') do: [ :var | self readVar: var toDepth: d ]	 ! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:43'!
setClass: aClass

	"This is the class the definition is originally specified on"
	theClass := aClass! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:51'!
setClassDefinition: aClassDefinition

	"this is the definition that this spec is being applied to, i.e. the 
	setClass and any subclasses which do not have their own spec"

	myDefinition := aClassDefinition! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:50'!
setDefinition: aClassDefinition

	"this is the definition that this spec is being applied to, i.e. the 
	setClass and any subclasses which do not have their own spec"

	myDefinition := aClassDefinition! !

!MagmaLocation methodsFor: '*magma seaside' stamp: 'kph 9/9/2006 03:21'!
newSessionAs: userName

 ^ self newSession connectAs: userName; yourself! !

!MaObjectRepository methodsFor: '*magma seaside-override' stamp: 'kph 6/14/2007 00:01'!
graphBufferAt: oidInteger using: aMaReadStrategy
	"Answers a MaSerializedGraphBuffer."

	| answer rootBuffer |
	answer _ MaSerializedGraphBuffer new: 400.
	rootBuffer _ 
		filer
			appendObject: oidInteger
			into: answer.

	rootBuffer
		appendGraphUsing: (aMaReadStrategy signalBeginningToRead: oidInteger classId: rootBuffer classId)
		into: answer
		currentDepth: 1
		minDepth: aMaReadStrategy minimumDepth
		with: Set new
		filer: filer.
	^ answer! !

WADispatcherPlugin subclass: #WAMagmaDispatcherPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 02:35'!
controlPanelClass

	^ WAMagmaControlPanel! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 03:49'!
execute 

	self requestor call:  (self controlPanelClass forApplication: self target)! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/18/2007 11:45'!
isActive 
	^ target isApplication and: 
		[ target configuration allAttributes anySatisfy: [ :c | c group = #magma]  ]! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/11/2007 17:01'!
label

	^ 'Magma'! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/11/2007 16:57'!
priority

	^ 400! !

WASystemConfiguration subclass: #WAMagmaConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 6/12/2007 14:34'!
ancestors

	^ #() "Array with: (WARenderLoopConfiguration new)"! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 10/27/2007 04:54'!
attributeLocation

	^ WAMagmaLocationAttribute key: #location group: #magma! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 10/27/2007 04:54'!
attributeReadStrategy

	^ (WAListAttribute key: #readStrategy group: #magma)
		options: self readStrategyClasses; 
		yourself
		! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 10/27/2007 04:55'!
attributeSignalProxyMaterializations

	^ WABooleanAttribute key: #signalProxyMaterializations group: #magma
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 10/27/2007 04:56'!
attributes
	^ (self class selectors
		select: [ :each | each ~= #attributes
			and: [ each beginsWith: #attribute ] ])
		collect: [ :each | self perform: each ]! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'brp 11/4/2005 11:44'!
location

	^ MagmaLocalLocation default
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 9/20/2006 14:10'!
readStrategy 

^ WAMagmaReadStrategyDefault 
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 9/20/2006 14:12'!
readStrategyClasses 

^ WAMagmaReadStrategyDefault withAllSubclasses 
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'brp 10/28/2005 11:35'!
repositoryPath

	^ 'store'
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'brp 10/28/2005 11:24'!
sessionClass

	^ WAMagmaSession! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 14:12'!
signalProxyMaterializations 

	^ false! !

WASession subclass: #WAMagmaSession
	instanceVariableNames: 'magmaSession holder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaSession commentStamp: 'kph 9/29/2006 02:15' prior: 0!
Experimental: the holder instVar is available to put a repository object(s) into for the duration of the session, in order to prevent realised objects from being garbage collected. When the session is unregistered the holder is set to nil releasing the object.!

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 6/12/2007 06:58'!
buildReadStrategyOn: theReadStrategy

	| theRoot |

	((theRoot := self root) respondsTo: #buildReadStrategyOn:) 
		ifTrue: [ theRoot buildReadStrategyOn: theReadStrategy ].

	^theReadStrategy.
! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 17:41'!
commit: aBlock

	self magmaSession commit: aBlock
! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/29/2006 02:15'!
holder

^holder! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/29/2006 02:15'!
holder: anObject

holder := anObject! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/9/2006 03:15'!
location 

^self application preferenceAt: #location.
! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/21/2006 06:11'!
log: message

	|timeStamp |					
		
	timeStamp := Time millisecondClockValue. 

^self readStrategyClass isLogger ifTrue: [ WAMagmaLog log entry: 'seaside' context: thisContext ]
 ! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 14:29'!
magmaMaterializing: object! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 6/14/2007 07:28'!
magmaSession

^ magmaSession ifNil: [ self obtainMagmaSession ]! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 6/18/2007 08:25'!
obtainMagmaSession
 
	self log: 'Obtaining MagmaSession'.

	magmaSession := self location newSessionAs: 'seaside'.

	magmaSession preferences signalProxyMaterializations: (self signalProxyMaterializations).
	self updateReadStrategy.

	^ magmaSession! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/17/2006 16:56'!
readStrategyClass

 ^ self application preferenceAt: #readStrategy! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 8/11/2006 03:41'!
repositoryPath
	"This is a convenience method to get the repository directory path from Seaside"

	^ self application preferenceAt: #repositoryPath! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'brp 10/28/2005 12:52'!
root

	^ self magmaSession root
! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 10/6/2006 04:19'!
setMagmaSession: ms

magmaSession := ms! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 14:06'!
setPreferencesFor: aMagmaSession

	aMagmaSession preferences signalProxyMaterializations: (self signalProxyMaterializations)! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 14:06'!
signalProxyMaterializations

 ^ self application preferenceAt: #signalProxyMaterializations! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 10/6/2006 05:00'!
stubOut: anObject

	self magmaSession stubOut: anObject
! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 9/28/2006 21:23'!
unregistered

holder := nil.
magmaSession ifNotNil: [ magmaSession disconnect. magmaSession := nil ]! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 6/18/2007 08:29'!
updateReadStrategy

	 ^ magmaSession readStrategy: 
		(self readStrategyClass setStrategyUsing: self in:  magmaSession) ! !

!WAMagmaSession methodsFor: 'as yet unclassified' stamp: 'kph 10/2/2007 23:45'!
withErrorHandler: aBlock
	^ [ super withErrorHandler: aBlock ] 
		on: MagmaProxyMaterialization 
		do: [ :n | self magmaMaterializing: n materializedObject. n resume ].! !

WAMagmaSession subclass: #WAMagmaSharedSession
	instanceVariableNames: ''
	classVariableNames: 'MagmaSessions'
	poolDictionaries: ''
	category: 'Magma seaside'!

!WAMagmaSharedSession commentStamp: 'kph 9/17/2006 02:42' prior: 0!
WAMagmaSharedSession keeps one session per repository, and this is provided to each Seaside Session. i.e. each seaside session can access the database but needs to be aware that it may be sharing its view of the model with other seaside sessions.

Pier has a model locking mechanism in place to prevent concurrent edits, and so will perform better with WAMagmaSharedSession than with a standard WAMagmaSession Since sharing the magma session avoids the overhead of connecting to the database at the beginning of every user session.

Rules for use:
Do not abort, you may ruin someone else's transaction.
Ensure that commits are negotiated amongst the sharing users via locking. If one seaside session commits: while another is in mid transaction their changes will be lost. 
To use a shared session there should be one data model that manages commits: and aborts on behalf of all of its seaside session clients.
!

!WAMagmaSharedSession class methodsFor: 'class initialization' stamp: 'kph 9/9/2006 03:32'!
initialize

MagmaSessions := Dictionary new.! !

WAMagmaSharedSession subclass: #CTMagmaSharedSession
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seaside'!

!CTMagmaSharedSession methodsFor: 'processing' stamp: 'lr 8/30/2006 12:04'!
cometRequest: aRequest
	^ (handlers
		at: (aRequest fields
			at: CTComet headerField
			ifAbsent: [ ^ nil ])
		ifAbsent: [ ^ nil ])
			connect: aRequest! !

!CTMagmaSharedSession methodsFor: 'processing' stamp: 'lr 7/12/2006 12:10'!
incomingRequest: aRequest
	aRequest responseStream
		ifNil: [ self error: 'Streaming-server required.' ].
	^ (aRequest fields includesKey: CTComet headerField)
		ifTrue: [ self cometRequest: aRequest ]
		ifFalse: [ super incomingRequest: aRequest ]! !

!CTMagmaSharedSession methodsFor: 'initialization' stamp: 'lr 7/12/2006 11:02'!
initializeWithApplication: anApplication
	super initializeWithApplication: anApplication.
	handlers := Dictionary new! !

!CTMagmaSharedSession methodsFor: 'public' stamp: 'lr 8/30/2006 11:57'!
registerPusher: aPusher context: aContext
	| handler |
	handler := CTHandler on: aPusher session: self context: aContext.
	handlers at: handler id put: handler.
	^ handler id! !

!WAMagmaSharedSession class methodsFor: 'class initialization' stamp: 'kph 9/19/2006 13:20'!
magmaSessions

^MagmaSessions

"self magmaSessions"! !

!WAMagmaSharedSession methodsFor: 'as yet unclassified' stamp: 'kph 9/17/2006 17:14'!
magmaAbortOnRequest.

"dont for a shared session"! !

!WAMagmaSharedSession methodsFor: 'as yet unclassified' stamp: 'kph 6/18/2007 08:36'!
obtainMagmaSession

"super obtainMagmaSession calls updateReadStrategy which assumes the magmaSession is stored 
in the attribute, in this cas it is not, so we fake it temporarily (not thread safe)"

| ms |

magmaSession := MagmaSessions at: (self location) ifAbsent: [ nil ].

((magmaSession isNil or: [ magmaSession isOpen not ]) or: [ magmaSession isConnected not]) 
	ifTrue: [ magmaSession := MagmaSessions at: self location put: super obtainMagmaSession ].

ms := magmaSession.
magmaSession := nil.

^ ms! !

!WAMagmaSharedSession methodsFor: 'as yet unclassified' stamp: 'kph 9/24/2006 04:18'!
unregistered

	magmaSession ifNotNil: [ magmaSession := nil ]! !

!UnixFileDirectory methodsFor: '*magma seaside' stamp: 'kph 9/1/2006 01:38'!
directoryExists: filenameOrPath
    "Handles the special case of testing for the root dir: there isn't a
    possibility to express the root dir as full pathname like '/foo'."

    ^ filenameOrPath = '/'
        or: [ filenameOrPath = self class default pathName
        or: [ super directoryExists: filenameOrPath ] ]
! !
WAMagmaControlPanel initialize!
WAMagmaSharedSession initialize!
